// FILE: node1.cxx
// IMPLEMENTS: The functions of the node class and the
// linked list toolkit (see node1.h for documentation).
// INVARIANT for the node class:
//   The data of a node is stored in data_field, and the link in link_field.

#include "node.h"
#include <cassert>    // Provides assert
#include <cstdlib>    // Provides NULL and size_t
using namespace std;


size_t list_length(const node* head_ptr)
// Library facilities used: cstdlib
{
    const node *cursor;
    size_t answer;
    
    answer = 0;
    for (cursor = head_ptr; cursor != NULL; cursor = cursor->link( ))
	++answer;
    
    return answer;
}

void list_head_insert(node*& head_ptr, value_type entry)
{
    head_ptr = new node(entry, head_ptr);
}

void list_insert(node*& head_ptr, node* previous_ptr, value_type entry)
{
    node *insert_ptr;		// Points to new node
    
    insert_ptr = new node(entry);
    if (previous_ptr != NULL)
    {
	insert_ptr->set_link(previous_ptr-> link());
	previous_ptr->set_link(insert_ptr);
    }
    else        // inserting at head of list
    {
	insert_ptr->set_link(head_ptr);
	head_ptr = insert_ptr;
    }
    
}

bool list_search(node* head_ptr, node*& cur_ptr, node*& previous_ptr, value_type target)
// Library facilities used: cstdlib
{
    bool found = false;
    
    cur_ptr = head_ptr;
    previous_ptr = NULL;
    
    while ((!found) && (cur_ptr != NULL) && (cur_ptr->data() <= target))
    {
	if (cur_ptr->data() == target)
	{
	    found = true;
	}
	else
	{
	    previous_ptr = cur_ptr;
	    cur_ptr = cur_ptr->link();
	}
    }
    return found;
}

bool list_search(const node* head_ptr, const node*& cur_ptr, const node*& previous_ptr, value_type target)
// Library facilities used: cstdlib
{
    bool found = false;
    
    cur_ptr = head_ptr;
    previous_ptr = NULL;
    
    while ((!found) && (cur_ptr != NULL) && (cur_ptr->data() <= target))
    {
	if (cur_ptr->data() == target)
	{
	    found = true;
	}
	else
	{
	    previous_ptr = cur_ptr;
	    cur_ptr = cur_ptr->link();
	}
    }
    return found;
}

node* list_locate(node* head_ptr, size_t position)
// Library facilities used: cassert, cstdlib
{
    node *cursor;
    size_t i;
    
    assert (0 < position);
    cursor = head_ptr;
    for (i = 1; (i < position) && (cursor != NULL); i++)
	cursor = cursor->link( );
    return cursor;
}

const node* list_locate(const node* head_ptr, size_t position)
// Library facilities used: cassert, cstdlib
{
    const node *cursor;
    size_t i;
    
    assert (0 < position);
    cursor = head_ptr;
    for (i = 1; (i < position) && (cursor != NULL); i++)
	cursor = cursor->link( );
    return cursor;
}

void list_head_remove(node*& head_ptr)
{
    node *remove_ptr;
    
    remove_ptr = head_ptr;
    head_ptr = head_ptr->link( );
    delete remove_ptr;
}

void list_remove(node* previous_ptr)
{
    node *remove_ptr;
    
    remove_ptr = previous_ptr->link( );
    previous_ptr->set_link( remove_ptr->link( ) );
    delete remove_ptr;
}

void list_clear(node*& head_ptr)
// Library facilities used: cstdlib
{
    while (head_ptr != NULL)
	list_head_remove(head_ptr);
}

void list_copy(const node* source_ptr, node*& head_ptr, node*& tail_ptr)
// Library facilities used: cstdlib
{
    head_ptr = NULL;
    tail_ptr = NULL;
    
    node *previous_ptr;
    
    // Handle the case of the empty list.
    if (source_ptr == NULL)
	return;
    
    // Make the head node for the newly created list, and put data in it.
    list_head_insert(head_ptr, source_ptr->data( ));
    tail_ptr = head_ptr;
    
    // Copy the rest of the nodes one at a time, adding at the tail of new list.
    source_ptr = source_ptr->link( );
    while (source_ptr != NULL)
    {
	list_insert(tail_ptr, previous_ptr, source_ptr->data( ));
	tail_ptr = tail_ptr->link( );
	source_ptr = source_ptr->link( );
    }
}

