/* Program: a59h5g2.cpp
 * By:  Tony G based on Craig Wrights
 * Date: 31/5/99
 *
 * Menu system that maintains a Binary Search Tree of integers.
 * - in increasing order.
 *
 */

#include <fstream.h>
#include <conio.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>

const int preorder = 1, inorder = 2, postorder = 3;

class BST
 {
  private:
	struct node
	 {
	  char name[30];
	  node *right, *left;
	 };

	node *root, *current, *take_away;
	fstream file;

	void display_nodes (node* p, int level, int horiz);
	int is_empty (void);
	void find_key (char wanted[], int& found);
	void sub_del (node*& q);
	void del (char elem[], node*& p, int& found);
	void write_to_file (node* p);

  public:
	BST() {root = NULL;}
	void display (void);
	void insert (char new_name[], int& inserted);
	void del_elem (char elem[], int& deleted);
	void store (void);
   void load (void);
 }; // BST

void main(void)
 {
  int choice, inserted, deleted;
  char name[30];
  char phone[30];
  BST tree;
  tree.load();
  do
	{
	 tree.display();
	 cout << "BINARY SEARCH TREE MENU SYSTEM:\n\n";
	 cout << "1. Place a new name and a phone number into the tree\n";
	 cout << "2. Remove a name and a phone number from the tree\n";
	 cout << "3. Exit the program.\n\n";
	 cout << "Choose an option [1-3]: ";
	 cin >> choice;
	 switch (choice)
	  {
		case 1: cout << "\nEnter a new name: ";
				  gets(name);
              cout << "\nEnter a corresponding phone-number: ";
              gets(phone);
              strcat(name,phone);
				  tree.insert (name, inserted);
				  if (inserted)
					{
					 cout << "\nNew name successfully added to the tree.\n";
                getch();
					}
				  else
					{
					 cout << "\nSorry, cannot add name. Tree is full.\n";
                getch();
					} // if
				  break;
		case 2: cout << "\nEnter a name: ";
				  cin >> name;
				  tree.del_elem (name, deleted);
				  if (deleted)
					{
					 cout << "\nName successfully deleted.\n";
					}
				  else
					{
					 cout << "\nName does not exist in tree.\n";
					} // if
				  break;
	  } // switch
	}
  while (choice != 3);
  tree.store();
 } // main

void BST::write_to_file (node* p)
// uses the pre-order traversal to write each node to file
 {
  if (!(p == NULL))
	{
	 file << p->name << endl;
	 write_to_file (p->left);
	 write_to_file (p->right);
	 delete p;
	} // if
 } // write_to_file

void BST::store (void)
// stores contents of tree in file
 {
  file.open ("a59ass4.dat", ios::out);
  write_to_file (root);
  root = NULL;
  current = NULL;
  file.close();
 } // store

void BST::display_nodes (node* p, int level, int horiz)
// uses the in-order traversal to display the contents of the tree
 {
  if (p != NULL)
	{
	 display_nodes (p->left, level + 1, horiz - 10/level);
	 gotoxy (horiz, level);
	 cout << p->name;
	 display_nodes (p->right, level + 1, horiz + 10/level);
	} // if
 } // display_nodes

void BST::display (void)
// public function to display tree
 {
  clrscr();
  cout << "The tree is (ordered listing)...";
  display_nodes (root, 1, 40);
  gotoxy (1, 10);
 } // display

int BST::is_empty (void)
// checks whether tree has no names in it
 {
  return (root == NULL);
 } // is_empty

void BST::find_key (char wanted[], int& found)
// searches for the element "wanted" in the tree
 {
  node *p, *prior;

  found = 0;
  if (!is_empty ())
	{
	 p = root;
	 do
	  {
		prior = p;
		if (strcmp(wanted,p->name)==0)
		 {
		  current = p;
		  found = 1;
		 }
		else
		 {
		  if (strcmp(wanted,p->name)<0)
			{
			 p = p->left;
			}
		  else
			{
			 p = p->right;
			} // if
		 } // if
	  }
	 while (!found && p != NULL);
	 if (!found)
	  {
		current = prior;
	  } // if
	} // if
 } // find_key

void BST::insert (char new_name[], int& inserted)
/* places "new_num" in the appropriate location for a binary
 * tree in ascending order
 */
 {
  node *p, *p_save;
  int found;

  inserted = 0;
  p_save = current;
  find_key (new_name, found);
  if (found)
	{
	 current = p_save;
	}
  else
	{
	 p = new node;
	 p->left = p->right = NULL;
	 strcpy(p->name,new_name);
	 if (is_empty())
	  {
		root = p;
	  }
	 else
	  {
		if (strcmp(new_name,current->name)<0)
		 {
		  current->left = p;
		 }
		else
		 {
		  current->right = p;
		 } // if
	  } // if
	 current = p;
	 inserted = 1;
	} // if
 } // insert

void BST::load (void)
// reads the tree in from its file
 {
  char new_name[30];
  int new_phone;
  int inserted;

  file.open ("a59ass4.dat", ios::in);
  if (file)
	{
	 while (!file.eof())
	  {
		file >> new_name;
		insert (new_name, inserted);
	  } // while
	 file.close();
	} // if
 } // load

void BST::sub_del (node*& q)
/* finds the next node that has no children to its right
 * isolates that node
 */
 {
  if (q->right != NULL)
	{
	 sub_del (q->right);
	}
  else
	{
    strcpy(take_away->name,q->name);
	 take_away = q;
	 q = q->left;
	} // if
 } // del_sub

void BST::del (char elem[], node*& p, int& found)
// finds the element to be deleted and deletes it
 {
  if (p == NULL)
	{
	 found = 0;
	}
  else
	{
	 if (strcmp(elem,p->name)<0)
	  {
		del (elem, p->left, found);
	  }
	 else
	  {
		if (strcmp(elem,p->name)>0)
		 {
		  del (elem, p->right, found);
		 }
		else
		 {
		  take_away = p;
		  if (p->right == NULL)
			{
			 p = p->left;
			}
		  else
			{
			 if (p->left == NULL)
			  {
				p = p->right;
			  }
			 else
			  {
				sub_del (p->left);
			  } // if
			} // if
		  delete take_away;
		  found = 1;
		 } // if
	  } // if
	} // if
 } // del

void BST::del_elem (char elem[], int& deleted)
/* finds the element to be deleted, deletes it and re-adjusts
 * if the tree becomes empty.
 */
 {
  deleted = 0;
  del (elem, root, deleted);
  if (!is_empty())
	{
	 current = root;
	} // if
 } // delete


