/////////////////////////////////////////////////////////////////////////////
// Robert Snyder 5/17/01
// heap.h - Heap function definitions.
//
// Functions:
//			 reheapUP(int,int) - Used for insertion on the heap.
//           reheapDown(int [], int, int) - Used for deletion from the heap.
//           Insert(int [], int &, int) - Inserts an item onto the heap.
//           Delete(int [], int &, int &) - Deletes an item from the heap.
//           random() - random number generator.
/////////////////////////////////////////////////////////////////////////////

#ifndef HEAP_H
#define HEAP_H

#include <stdlib.h> // Needed for srand() & rand().

#define TRUE 1	// Define a generic boolean data type.
#define FALSE 0

const int HEAP_SIZE = 10000; // The maximum size of the heap.

// Move the item up the heap until it finds its correct position.
void reheapUp(int heap[],int newPos)
{
	int parent;
	int hold;

	if(newPos) // newPos not zero
	{
		parent = (newPos-1)/2;
		if(heap[newPos] < heap[parent]) // Make it a min-heap.
		{                               // Parent less than or
			hold = heap[parent];        // equal to children.
			heap[parent] = heap[newPos];
			heap[newPos] = hold;
			reheapUp(heap,parent);
		}
	}
}

// Move the item down the heap until it finds its correct place.
void reheapDown(int heap[], int root, int last)
{
	int hold;
	int left_val;
	int right_val;
	int small_child_index;

	if((root*2+1) <= last)
	{
		left_val = heap[root*2+1];
		
		if((root*2+1) <= last)
			right_val = heap[root*2+2];
		else
			right_val = -1;

		if(left_val < right_val) // Make it a min-heap.
			small_child_index = root*2+1;
		else
			small_child_index = root*2+2;

		if(heap[root] > heap[small_child_index]) // Make it a min-heap.
		{
			hold = heap[root];
			heap[root] = heap[small_child_index];
			heap[small_child_index] = hold;
			reheapDown(heap,small_child_index,last);
		}
	}
}

// Insert an item into the heap.
int Insert(int heap[], int &last, int data)
{
	if(last == HEAP_SIZE-1)
		return FALSE;

	++last;
	heap[last] = data;
	reheapUp(heap,last);
	return TRUE;

}

// Delete an item from the heap.
int Delete(int heap[], int &last, int &val) // return the root value
{											// and remove it.
	if(last < 0)
		return FALSE;

	val = heap[0];
	heap[0] = heap[last];
	last--;
	reheapDown(heap,0,last);
	return TRUE;

}

// Random number generator.
int random()
{
 static int initialize = 1;

 if(initialize)
 {
  srand(1);
  initialize--;
 }

 return ((rand() %10000) + 1);

}

#endif // End ifndef