//////////////////////////////////////////////////////////////////////////
// GrowArray.h
//////////////////////////////////////////////////////////////////////////
// Declaration of the GrowArray template class. This is a helper class
// which is basically a simple Array class that dynamically resizes itself
// if you attempt to alter an index beyond its current size using the
// subscript operator.
// This is the data type that holds the value of a HUGE object's value
// and why they are limitless.
//////////////////////////////////////////////////////////////////////////
// This is free source. It may be a bit faulty. I haven't tested it to
// any real extremes. If you expand this source, please send me your
// version. I'd appreciate it.
//////////////////////////////////////////////////////////////////////////
// 08/08/02 - ROMMEL SANTOR - lucky760@yahoo.com
//////////////////////////////////////////////////////////////////////////

#ifndef __INCLUDED_GROW_ARRAY_H__
#define __INCLUDED_GROW_ARRAY_H__

#define DEF_SIZE  30

template <typename T>
class GrowArray {
protected:  
  int m_nSize;
  T*  m_Data;
public:
  GrowArray(int nSize=DEF_SIZE)
  : m_nSize(nSize), m_Data(new T[nSize])
  { }
  //GrowArray(const GrowArray &ga) { operator=(ga); }  
  virtual ~GrowArray() { delete [] m_Data; }
  
  GrowArray& operator=(const GrowArray<T> &ga);
    
  int GetSize() const { return m_nSize; }
  T*  GetData() const { return m_Data; }
    
  void Resize(int nSize);    
  T& operator[](int nPos);// { assert(nPos < m_nSize); return m_Data[nPos]; }
  const T& operator[](int nPos) const { assert(nPos < m_nSize); return m_Data[nPos]; }
}; 


template <typename T>
GrowArray<T>& GrowArray<T>::operator=(const GrowArray<T> &ga) {
  Resize(ga.m_nSize);
  for (int i=0;i<m_nSize;++i)
    m_Data[i] = ga[i];
  return *this;
}


// GrowableArray template member functions here
template <typename T>
void GrowArray<T>::Resize(int nSize) {
  if (nSize == m_nSize)
    return;
  if (nSize < m_nSize) {
    T *temp = new T[nSize];
    for (int i=0;i<nSize;++i)
      temp[i] = m_Data[i];
    delete [] m_Data;
    m_Data = temp;
  }
  else if (nSize > m_nSize) {
    T *temp = new T[nSize];
    for (int i=0;i<nSize;++i)
      temp[i] = i<m_nSize ? m_Data[i] : T();
    delete [] m_Data;
    m_Data = temp;
  }
  m_nSize = nSize;
}


template <typename T>
T& GrowArray<T>::operator[](int nPos) {
  if ((nPos+1) > m_nSize)
    Resize(m_nSize*2);
  return m_Data[nPos];
}


#endif //__INCLUDED_GROW_ARRAY_H__
