//////////////////////////////////////////////////////////////////////////
// HugeInt.h
//////////////////////////////////////////////////////////////////////////
// Declaration of the Huge Integer class. This is a data type that
// supports unlimited sized numeric values. Think of it as an integer that
// can be ANY value, no matter how large.
// All standard math operators are applicable (+ - * / %)
// All logical operators apply (> < >= <= == !=)
// Conversion to HINT is possible from the following types:
//   int, double, const char*
// Standard insertion and extraction (<< and >>) operators also apply.
// Use HINT as you would a normal int variable. The difference is that
// you can assign it a string (if you want to assign a huge number that's
// larger than the largest built-in variable). For example:
//   HINT huge = "999000123000999888777666555444333222111987654321888666"
//   cout << huge * "99999999999999999" << endl;
// If you cast or assign a double precision value, any decimal value will
// be truncated and not rounded.
//////////////////////////////////////////////////////////////////////////
// This is free source. It may be a bit faulty. I haven't tested it to
// any real extremes. If you expand this source, please send me your
// version. I'd appreciate it.
//////////////////////////////////////////////////////////////////////////
// 08/08/02 - ROMMEL SANTOR - lucky760@yahoo.com
//////////////////////////////////////////////////////////////////////////

#ifndef __INCLUDED_HUGE_INT_H__
#define __INCLUDED_HUGE_INT_H__

#include <windows.h>
#include <iostream>
#include <cassert>
#include "GrowArray.h"


typedef GrowArray<char> CHARR;


class HINT {//Huge Integer
  CHARR m_Digits;
  bool  m_bNeg;
  
  void RemoveLeadingZeros();
public:
  /////////////////////////////////////////////////////
  // Ctors & Dtor
  HINT() : m_bNeg(false) { }
  HINT(int nVal)          { operator=(nVal); }
  HINT(const char *szVal) { operator=(szVal); }
  HINT(const HINT &rhs)   { operator=(rhs); }
  HINT(double dVal)       { operator=(dVal); }// decimal is truncated, not rounded
  virtual ~HINT() { }
  
  /////////////////////////////////////////////////////
  // Assignment Operators
  HINT&         operator=(const char *szVal);
  HINT&         operator=(int nVal);
  HINT&         operator=(const HINT &rhs);
  HINT&         operator=(double dVal);

  /////////////////////////////////////////////////////
  // Logical Operators
  bool          operator>(const HINT &rhs) const;  
  bool          operator==(const HINT &rhs) const;
    
  bool          operator>=(const HINT &rhs) const { return operator>(rhs) || operator==(rhs); }  
  bool          operator<(const HINT &rhs) const { return !operator>=(rhs); }
  bool          operator<=(const HINT &rhs) const { return !operator>(rhs); }  
  bool          operator!=(const HINT &rhs) const { return !operator==(rhs); }

  /////////////////////////////////////////////////////
  // Mathematical Operators
  const HINT    operator+(const HINT &rhs) const;
  const HINT    operator-(const HINT &rhs) const;
  const HINT    operator*(const HINT &rhs) const;
  const HINT    operator/(const HINT &numer) const;
  const HINT    operator%(const HINT &rhs) const;
  
  HINT&         operator-=(const HINT &rhs) { return operator=(operator-(rhs)); }
  HINT&         operator+=(const HINT &rhs) { return operator=(operator+(rhs)); }
  HINT&         operator*=(const HINT &rhs) { return operator=(operator*(rhs)); }
  HINT&         operator/=(const HINT &rhs) { return operator=(operator/(rhs)); }
  HINT&         operator%=(const HINT &rhs) { return operator=(operator%(rhs)); }
  
  /////////////////////////////////////////////////////
  // Negative operator
  const HINT    operator-();//ie: someVal = someNum * (-hint);
  
  /////////////////////////////////////////////////////
  // Increment/Decrement operators
  HINT&         operator++();//prefix
  HINT&         operator--();//prefix
  const HINT    operator++(int);//postfix
  const HINT    operator--(int);//postfix 

  /////////////////////////////////////////////////////
  // Conversion
  char*         HToStr(char *szDest,int nMaxLen);//nMaxLen is max size of szDest excluding the NULL

  /////////////////////////////////////////////////////
  // Other Member Funcs  
  void Alloc(int nSize) { m_Digits.Resize(nSize); ZeroMemory(m_Digits.GetData(),m_Digits.GetSize()); }
  
  /////////////////////////////////////////////////////
  // Friends
  friend std::ostream&    operator<<(std::ostream &out,const HINT &rhs);
  friend std::istream&    operator>>(std::istream &in,HINT &rhs);
  
  friend const HINT       operator-(int nVal,const HINT &rhs);
  friend const HINT       operator+(int nVal,const HINT &rhs);
  friend const HINT       operator*(int nVal,const HINT &rhs);
  friend const HINT       operator/(int nVal,const HINT &rhs);
  friend const HINT       operator%(int nVal,const HINT &rhs);
  
  friend const HINT       operator-(const char *szVal,const HINT &rhs);
  friend const HINT       operator+(const char *szVal,const HINT &rhs);
  friend const HINT       operator*(const char *szVal,const HINT &rhs);
  friend const HINT       operator/(const char *szVal,const HINT &rhs);
  friend const HINT       operator%(const char *szVal,const HINT &rhs);
};

///////////////////////////////////////////////////////////////////
// FREE FUNCTIONS
std::istream& getline(std::istream &in,HINT &rhs,char delim='\n');

#endif //__INCLUDED_HUGE_INT_H__
